/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is ForumZilla 0.1.1.
 *
 * The Initial Developer of the Original Code is Zapogee, Inc.
 * Portions created by Zapogee, Inc. are Copyright (C) 2001
 * Zapogee, Inc.  All Rights Reserved.
 *
 * Contributor(s):
 *   Myk Melez, <myk@melez.com>, original author
 */

/* global variables */

var feedService = new FeedService();



/* load, unload, and selection handlers */

function doLoad() {
    updateModeButton();

    // Display the list of feed subscriptions.
    var ds = feedService.subscriptionsDS;
    var tree = document.getElementById('subscriptions');
    tree.database.AddDataSource(ds);
    tree.builder.rebuild();
    if (tree.view && tree.view.rowCount > 0)
        tree.view.selection.select(0);
    tree.focus();

    initializeFilterWidgets();
}

function doClose() {
    window.close();
}

function doSelect(event) {
    var tree = document.getElementById('subscriptions');
    var titleField = document.getElementById('title');
    var urlField = document.getElementById('url');
    var folderField = document.getElementById('actionTargetFolder');
    var intervalField = document.getElementById('interval');
    var quickModeField = document.getElementById('quickmode');
    var enabledField = document.getElementById('enabled');
    var editCaption = document.getElementById('edit-caption');

    if (tree.view.selection.currentIndex == -1) {
        titleField.value = "";
        urlField.value = "";
        SetFolderPicker(feedService.storageFolder, 'actionTargetFolder');
        intervalField.value = "";
        quickModeField.selectedIndex = -1;
        enabledField.checked = false;
        editCaption.setAttribute('label', "Edit Feed");

        //fooField.setAttribute('disabled', true); ???
    }
    else {
        var item = tree.view.getItemAtIndex(tree.view.selection.currentIndex);
        var resource = rdf.GetResource(item.id);
        if (!item.feed)
            item.feed = new Feed(resource);

        //fooField.setAttribute('disabled', false); ???

        titleField.value = item.feed.title;
        urlField.value = item.feed.url;
        SetFolderPicker(item.feed.folder.URI, 'actionTargetFolder');
        intervalField.value = item.feed.interval;
        quickModeField.selectedIndex = item.feed.quickMode ? 1 : 0;
        enabledField.checked = item.feed.enabled ? true : false;
        editCaption.setAttribute('label', "Edit " + item.feed.title);
    }
}



/* button handlers */

function doAdd() {
    var tree = document.getElementById('subscriptions');
    var url = window.prompt("Enter feed location:", "");
    if (url == null)
        return;
    add(url);
}

function doRemove() {
    var tree = document.getElementById('subscriptions');
    var selectedIndex = tree.view.selection.currentIndex;
    var treeitem = tree.view.getItemAtIndex(selectedIndex);
    var feed = rdf.GetResource(treeitem.id);

    // Remove the feed from the list of subscriptions.
    var feeds = feedService.subscriptionsList;
    var index = feeds.IndexOf(feed);
    if (index != -1)
        feeds.RemoveElementAt(index, false);

    // Remove all assertions about the feed from the subscriptions database.
    var ds = feedService.subscriptionsDS;
    removeAssertions(ds, feed);

    // Remove all assertions about items in the feed from the items database.
    ds = feedService.itemsDS;
    var items = ds.GetSources(FZ_FEED, feed, true);
    while (items.hasMoreElements()) {
        var item = items.getNext();
        item = item.QueryInterface(Components.interfaces.nsIRDFResource);
        ds.Unassert(item, FZ_FEED, feed, true);
        if (ds.hasArcOut(item, FZ_FEED))
            debug(item.Value + " is from more than one feed; only the reference to this feed removed");
        else
            removeAssertions(ds, item);
    }

    tree.builder.rebuild();
    if (tree.view.rowCount > selectedIndex)
        tree.view.selection.select(selectedIndex);
    else if (tree.view.rowCount > 0)
        tree.view.selection.select(tree.view.rowCount-1);
    tree.focus();
}

function doCheck() {
    var tree = document.getElementById('subscriptions');
    downloadFeedAtIndex(tree.view.selection.currentIndex);
}

function toggleMode() {
    var advancedSection = document.getElementById('advanced-section');
    if (advancedSection.getAttribute('hidden') == 'true')
        advancedSection.setAttribute('hidden', 'false');
    else
        advancedSection.setAttribute('hidden', 'true');
    updateModeButton();
}

function doOpenPreferences() {
    window.openDialog('chrome://forumzilla/content/preferences.xul', '', 'chrome,dialog,modal');
}

function doImport() {
    var rv = pickOpen('Pick OPML File to Import', '$xml $all');
    if (rv.reason == PICK_CANCEL)
        return false;

    var file = new LocalFile(rv.file, MODE_RDONLY);
    var text = file.read();
    var gParser = new DOMParser;
    var xmlDocument = gParser.parseFromUTF8String(text, 'text/xml');
    var feeds = xmlDocument.getElementsByTagName("body")[0].getElementsByTagName("outline");
    for ( var i=0 ; i<feeds.length ; i++ ) {
        var feed = feeds[i];
        debug("importing from OPML feed " + feed.getAttribute("title"));
        if (feed.getAttribute("type") == "rss" && feed.getAttribute("title"))
            add(feed.getAttribute("xmlUrl"), feed.getAttribute("title"));
    }
    return true;
}



/* field update handlers */

function doUpdateURL() {
    var feed = getFeedForSelectedItem();
    feed.url = document.getElementById('url').value;
}

function doUpdateTitle() {
    var feed = getFeedForSelectedItem();
    feed.title = document.getElementById('title').value;
    document.getElementById('edit-caption').setAttribute('label', "Edit Feed: " + feed.title);
}

function doUpdateInterval() {
    var feed = getFeedForSelectedItem();
    feed.interval = document.getElementById('interval').value;
}

function doUpdateQuickMode() {
    var feed = getFeedForSelectedItem();
    feed.quickMode = document.getElementById('quickmode').value == "true" ? true : false;
}

function doUpdateEnabled() {
    var feed = getFeedForSelectedItem();
    feed.enabled = document.getElementById('enabled').checked;
}

function doUpdateFolder() {
    var feed = getFeedForSelectedItem();
    feed.folder = document.getElementById('actionTargetFolder').getAttribute('uri');

    // Rebuild the tree, since it doesn't seem to update otherwise;
    // maybe trees auto-rebuild for literals but not resources?
    var tree = document.getElementById('subscriptions');
    var index = tree.view.selection.currentIndex;
    tree.builder.rebuild();
    tree.view.selection.select(index);
}



/* misc */

function getFeedForSelectedItem() {
    var tree = document.getElementById('subscriptions');
    var item = tree.view.getItemAtIndex(tree.view.selection.currentIndex);
    var feed = item.feed || new Feed(rdf.GetResource(item.id));
    return feed;
}

function updateModeButton() {
    var advancedSection = document.getElementById('advanced-section');
    var modeButton = document.getElementById('mode-button');
    if (advancedSection.getAttribute('hidden') == 'true')
        modeButton.label = "Advanced...";
    else
        modeButton.label = "Simple...";
}

function downloadFeedAtIndex(index) {
    var tree = document.getElementById('subscriptions');
    var ds = feedService.subscriptionsDS;

    var item;
    try {
        item = tree.view.getItemAtIndex(index);
    } catch(e) {
        debug("couldn't get unselected item at index " + index);
        return;
    }
    var resource = rdf.GetResource(item.id);
    window.opener.feedService.downloadFeed(resource);
    //window.opener.feedService.storeItems();
}

function add(url, title) {
    var tree = document.getElementById('subscriptions');
    var ds = feedService.subscriptionsDS;
    var feeds = feedService.subscriptionsList;

    // Generate a unique ID for the feed.
    var id = url;
    var i = 1;
    while (feeds.IndexOf(rdf.GetResource(id)) != -1 && ++i < 1000)
        id = url + i;
    if (id == 1000)
        throw("couldn't generate a unique ID for feed " + url);

    // Create the feed and add it to the database.
    id = rdf.GetResource(id);
    feeds.AppendElement(id);
    feed = new Feed(id);
    ds.Assert(id, RDF_TYPE, FZ_FEED, true);

    // Store information about the feed.
    feed.url = url;
    // In theory it isn't necessary to store this information for the feed,
    // since it'll pick up the default values, but the subscriptions tree won't
    // display the default values, so it's useful to add them. XXX Figure out
    // a better solution, since this prevents the values from updating
    // automatically when the defaults get updated.
    feed.quickMode = false;
    feed.interval = feedService.pollingInterval;
    feed.enabled = true;

    if (title)
        feed.title = title;
    else {
        // Download the feed to pick up the title.  Note that this downloads
        // the feed synchronously, which causes the UI to freeze while the feed
        // is being downloaded.  XXX We should figure out a better thing
        // to do here, at least tell the user to wait while we get this info,
        // but it would be even better to download it asynchronously and only
        // create the folder once we've received the title.
        feed.download(false, false);
    }

    // We access the folder and then throw it away to trigger the folder getter
    // to create the folder for the feed.
    feed.folder;

    // Rebuild the tree, select the newly created feed, and focus the title field
    // so the user can edit it. XXX Don't focus the title field if we're in
    // simple mode.
    tree.builder.rebuild();
    tree.view.selection.select(tree.view.rowCount-1);
    document.getElementById('title').focus();
}
